﻿using System;
using System.IO;
using System.Text;
using System.Configuration;
using System.Net;
using System.Net.Sockets;
using System.Data;

using Ion.Core;
using Ion.Storage;
using Ion.Configuration;

namespace Ion
{
    /// <summary>
    /// A static class that holds references to all managers and registered instances.
    /// </summary>
    public class IonEnvironment
    {
        #region Fields
        private static Logging mLog = new Logging();
        private static ConfigurationModule mConfig;
        private static Encoding mTextEncoding = Encoding.GetEncoding(28591); // Latin1
        public static AsyncCallback pfnWorkerCallBack;
        public static Socket m_socListener;
        public static Socket m_socWorker;
        private static DatabaseManager mDatabaseManager;
        public static IAsyncResult asyn1;
        public static DateTime lstrec;

        #endregion

        #region Properties
        public static Logging Log
        {
            get { return mLog; }
        }
        public static ConfigurationModule Configuration
        {
            get { return mConfig; }
        }
        #endregion

        #region Methods
        /// <summary>
        /// Initializes the Ion server environment.
        /// </summary>
        public static void Initialize()
        {
            Log.minimumLogImportancy = LogType.Debug;
            Log.WriteLine("Initializing Ion environment.");

            try
            {
                // Try to initialize configuration
                try
                {
                    mConfig = ConfigurationModule.LoadFromFile("settings");
                }
                catch (FileNotFoundException ex)
                {
                    Log.WriteError("Failed to load configuration file, exception message was: " + ex.Message);
                    IonEnvironment.Destroy();
                    return;
                }

                // Initialize database and test a connection by getting & releasing it
                DatabaseServer pDatabaseServer = new DatabaseServer(
                    IonEnvironment.Configuration["db1.server.host"],
                    IonEnvironment.Configuration.TryParseUInt32("db1.server.port"),
                    IonEnvironment.Configuration["db1.server.uid"],
                    IonEnvironment.Configuration["db1.server.pwd"]);

                Database pDatabase = new Database(
                    IonEnvironment.Configuration["db1.name"],
                    IonEnvironment.Configuration.TryParseUInt32("db1.minpoolsize"),
                    IonEnvironment.Configuration.TryParseUInt32("db1.maxpoolsize"));

                mDatabaseManager = new DatabaseManager(pDatabaseServer, pDatabase);
                mDatabaseManager.SetClientAmount(2);
                mDatabaseManager.ReleaseClient(mDatabaseManager.GetClient().Handle);
                mDatabaseManager.StartMonitor();
                //create the listening socket...
                m_socListener = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                IPEndPoint ipLocal = new IPEndPoint(IPAddress.Any, 91);
                //bind to local IP Address...
                m_socListener.Bind(ipLocal);
                //start listening...
                m_socListener.Listen(4);
                // create the call back for any client connections...
                m_socListener.BeginAccept(new AsyncCallback(OnClientConnect), null);
                Log.WriteLine("Initialized Ion environment.");
            }
            catch (SocketException se)
            {
                Log.WriteError(se.Message);
            }
            catch (Exception ex) // Catch all other exceptions
            {
                mLog.WriteError("Unhandled exception occurred during initialization of Ion environment. Exception message: " + ex.Message);
            }
        }
        public static void OnClientConnect(IAsyncResult asyn)
        {
            try
            {
                m_socWorker = m_socListener.EndAccept(asyn);
                WaitForData(m_socWorker);
            }
            catch (ObjectDisposedException)
            {
                System.Diagnostics.Debugger.Log(0, "1", "\n OnClientConnection: Socket has been closed\n");
            }
            catch (SocketException se)
            {
                Log.WriteError(se.Message);
            }
        }
        public static void WaitForData(System.Net.Sockets.Socket soc)
        {
            try
            {                
             //   if (DateTime.Compare(lstrec, DateTime.Now) == 0)
               // {
                    if (pfnWorkerCallBack == null)
                    {
                        pfnWorkerCallBack = new AsyncCallback(OnDataReceived);
                    }
                    CSocketPacket theSocPkt = new CSocketPacket();
                    theSocPkt.thisSocket = soc;
                    // now start to listen for any data...
                    soc.BeginReceive(theSocPkt.dataBuffer, 0, theSocPkt.dataBuffer.Length, SocketFlags.None, pfnWorkerCallBack, theSocPkt);
                //}
               // else
                //{
                    Disconnect();
               // }
            }
            catch (SocketException se)
            {
                Log.WriteError(se.Message);
            }

        }

        public  static void OnDataReceived(IAsyncResult asyn)
        {
            try
            {
                CSocketPacket theSockId = (CSocketPacket)asyn.AsyncState;
                //end receive...
                int iRx = 0;
                iRx = theSockId.thisSocket.EndReceive(asyn);
                char[] chars = new char[iRx + 1];
                System.Text.Decoder d = System.Text.Encoding.UTF8.GetDecoder();
                int charLen = d.GetChars(theSockId.dataBuffer, 0, iRx, chars, 0);
                System.String szData = new System.String(chars);
                if ((szData == "") == false)
                    Recv(szData);
                WaitForData(m_socWorker);
            }
            catch (ObjectDisposedException)
            {
                System.Diagnostics.Debugger.Log(0, "1", "\nOnDataReceived: Socket has been closed\n");
            }
            catch (SocketException se)
            {
                Log.WriteError(se.Message);
            }
        }
        /// <summary>
        /// Destroys the Ion server environment and exits the application.
        /// </summary>
        public static void Recv(string D)
        {
            lstrec = DateTime.Now;
            string[] info = D.Split(' ');
            switch (info[2])
            {
                case "LOGIN":
                    {
                        Login(D, IonEnvironment.GetDatabase());
                        break;
                    }
                case "REGISTER":
                    {
                        Register(D, IonEnvironment.GetDatabase());
                        break;
                    }
                default:
                    {
                        Log.WriteError("Missed: " + D);
                        break;
                    }
            }
        }
        public static void Register(string D, DatabaseManager database)
        {
            // REGISTER <name> <password> <email> <pants>,<shirt>,<head> <conntype> x <sex> <spam me yes/no> <customData>
            string[] info = D.Split(' ');
            string[] look = info[6].Replace(",","").Split(' ');
            using (DatabaseClient dbClient = mDatabaseManager.GetClient())
            {
                dbClient.ExecuteQuery("INSERT INTO users ('username','password','email','pants','shirt','head','conntype','x','sex','motto') VALUES ('" + info[3] + "','" + info[4] + "','" + info[5] + "','" + look[0] + "','" + look[1] + "','" + look[2] + "','" + info[7] + "','x','0','" + info[10] + "')");
            }
        }
        public static void Login(string D, DatabaseManager database)
        {
            //LOGIN <username> <password>
            string[] info = D.Split(' ');
            string username = info[3];
            string password = info[4];
            using (DatabaseClient dbClient = mDatabaseManager.GetClient())
            {
                int uid = dbClient.ReadInt32("SELECT uid FROM users WHERE username = '" + username + "'");
                string dbpassword = dbClient.ReadString("SELECT password FROM users WHERE username = '" + username + "'");
                if (dbpassword == password)
                    Enter(IonEnvironment.GetDatabase());
                else
                {
                    CSocketPacket theSocPkt = new CSocketPacket();
                    Disconnect();
                }
            }
        }
        public static void Enter(DatabaseManager database)
        {

        }
        public static void Disconnect()
        {
            //CSocketPacket theSockId = (CSocketPacket)asyn1.AsyncState;
           // theSockId.thisSocket.Disconnect(false);
        }
        public static void Destroy()
        {
            Log.WriteLine("Destroying Ion environment.");

            // Stop database
            if (GetDatabase() != null)
            {
                IonEnvironment.Log.WriteLine("Destroying database " + GetDatabase().ToString());
                GetDatabase().StopMonitor();
                GetDatabase().DestroyClients();
                GetDatabase().DestroyManager();
            }

            Log.WriteLine("Press a key to exit.");

            Console.ReadKey();
            System.Environment.Exit(2);
        }

        /// <summary>
        /// Returns the configuration module.
        /// </summary>
        /// <returns>ConfigurationModule holding configuration values for Ion.</returns>
        public static ConfigurationModule GetConfiguration()
        {
            return mConfig;
        }
        /// <summary>
        /// Returns the default System.Text.Encoding for encoding and decoding text.
        /// </summary>
        /// <returns>System.Text.Encoding</returns>
        public static Encoding GetDefaultTextEncoding()
        {
            return mTextEncoding;
        }

        public static DatabaseManager GetDatabase()
        {
            return mDatabaseManager;
        }
        #endregion
		public class CSocketPacket
		{
			public System.Net.Sockets.Socket thisSocket;
			public byte[] dataBuffer = new byte[1024];
		}

		
    }
}
